--
-- Copyright (c) 2015 Aitia International Inc.
--
-- See the file "/doc/license.txt" for information on usage and redistribution
-- of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-- 

-- ARCH.VHD
-- ========
--
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.tuw.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

--
PACKAGE arch IS
--
  CONSTANT  aDefSize: INTEGER := 32;         -- Default size of logic vectors

--
  COMPONENT aRisingEdge
--          ~~~~~~~~~~~
        PORT (
             I:     IN     STD_LOGIC;       -- Input async/combi.
             C:     IN     STD_LOGIC;       -- Clock system
--
             Q:     OUT    STD_LOGIC        -- Pulse on Rising edge
             );
  END COMPONENT;
--
  COMPONENT aFallingEdge
--          ~~~~~~~~~~~
        PORT (
             I:     IN     STD_LOGIC;       -- Input async/combi.
             C:     IN     STD_LOGIC;       -- Clock system
             Q:     OUT    STD_LOGIC        -- Pulse on Falling edge
             );
  END COMPONENT;
--
  COMPONENT aRippleRise
--          ~~~~~~~~~~~
        PORT (
             I:     IN     STD_LOGIC;       -- Input sync. other clock
             C:     IN     STD_LOGIC;       -- Clock system
             Q:     OUT    STD_LOGIC        -- Pulse on Rising edge
             );
  END COMPONENT;
--
  COMPONENT aDelay
--          ~~~~~~
        GENERIC( Length: INTEGER := 1  );
        PORT (
             I:     IN     STD_LOGIC;      -- Input
             C:     IN     STD_LOGIC;      -- Clock
             CE:    IN     STD_LOGIC;      -- Enable
             Q:     OUT    STD_LOGIC       -- Output
             );
  END COMPONENT;
--
  COMPONENT aDelayx2
--          ~~~~~~
        GENERIC( Length: INTEGER := 1  );
        PORT (
             I:     IN     STD_LOGIC;      -- Input
             C:     IN     STD_LOGIC;      -- Clock
             CE:    IN     STD_LOGIC;      -- Enable
             Q:     OUT    STD_LOGIC       -- Output
             );
  END COMPONENT;

  COMPONENT aPipe
--          ~~~~~
        GENERIC( Size:   INTEGER := aDefSize ;
                 Length: INTEGER := 1  );
        PORT (
             I:     IN     STD_LOGIC_VECTOR (Size-1 downto 0); -- Input
             C:     IN     STD_LOGIC;                          -- Clock
             CE:    IN     STD_LOGIC;                          -- Enable
             Q15:   OUT    STD_LOGIC_VECTOR (Size-1 downto 0);  -- Output
             Q:     OUT    STD_LOGIC_VECTOR (Size-1 downto 0)  -- Output
             );
  END COMPONENT;
--
  COMPONENT aPipex2
--          ~~~~~
        GENERIC( Size:   INTEGER := aDefSize ;
                 Length: INTEGER := 1  );
        PORT (
             I:     IN     STD_LOGIC_VECTOR (Size-1 downto 0); -- Input
             C:     IN     STD_LOGIC;                          -- Clock
             CE:    IN     STD_LOGIC;                          -- Enable
             Q31:   OUT    STD_LOGIC_VECTOR (Size-1 downto 0);  -- Output
             Q:     OUT    STD_LOGIC_VECTOR (Size-1 downto 0)  -- Output
             );
  END COMPONENT;
--
  COMPONENT aProgPipe
--          ~~~~~~~~~
        GENERIC( Size:   INTEGER := aDefSize );
        PORT (
             I:     IN     STD_LOGIC_VECTOR (Size-1 downto 0); -- Input
             C:     IN     STD_LOGIC;                          -- Clock
             CE:    IN     STD_LOGIC;                          -- Enable
             A:     IN     STD_LOGIC_VECTOR (3 downto 0);      -- Addr
             Q:     OUT    STD_LOGIC_VECTOR (Size-1 downto 0)  -- Output
             );
  END COMPONENT;
--
  COMPONENT aRegister
--          ~~~~~~~~~
        GENERIC( Size:   INTEGER := aDefSize );
        PORT (
             D:     IN     STD_LOGIC_VECTOR (Size-1 downto 0); -- Input
             C:     IN     STD_LOGIC;                          -- Clock
             CE:    IN     STD_LOGIC;                          -- Enable
             Q:     OUT    STD_LOGIC_VECTOR (Size-1 downto 0)  -- Output
             );
  END COMPONENT;
--
COMPONENT aRAM
--        ~~~~~~~~~~
        GENERIC( Size: INTEGER := aDefSize );
        PORT (
              D     : IN  STD_LOGIC_VECTOR( Size-1 downto 0 ); -- Data in
              WE    : IN  STD_LOGIC;       -- Write enable
              WCLK  : IN  STD_LOGIC;       -- Clock
              A     : IN  STD_LOGIC_VECTOR ( 3 downto 0);      -- Write Address
--
              RA    : IN  STD_LOGIC_VECTOR ( 3 downto 0);      -- Read Address
              DO    : OUT STD_LOGIC_VECTOR( Size-1 downto 0 )  -- Data out
             );
END COMPONENT;
--
COMPONENT aRAMx2
--        ~~~~~~~~~~
        GENERIC( Size: INTEGER := aDefSize );
        PORT (
              D     : IN  STD_LOGIC_VECTOR( Size-1 downto 0 ); -- Data in
              WE    : IN  STD_LOGIC;       -- Write enable
              WCLK  : IN  STD_LOGIC;       -- Clock
              A     : IN  STD_LOGIC_VECTOR ( 4 downto 0);      -- Write Address
--
              RA    : IN  STD_LOGIC_VECTOR ( 4 downto 0);      -- Read Address
              DO    : OUT STD_LOGIC_VECTOR( Size-1 downto 0 )  -- Data out
             );
END COMPONENT;
--
END arch;
--
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.arch.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY      aRisingEdge IS
--          ~~~~~~~~~~~
        PORT (
             I:     IN     STD_LOGIC;
             C:     IN     STD_LOGIC;
--
             Q:     OUT    STD_LOGIC ); -- Out
END         aRisingEdge;
--
ARCHITECTURE MacroCellRE  OF aRisingEdge IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~~
SIGNAL       Ta,Tb,Tc    : STD_LOGIC;
SIGNAL       Low,Hi      : STD_LOGIC;
--
BEGIN
--
Low <= '0';
Hi  <= '1';
Tc  <= I AND Ta AND NOT Tb;
--
U0:    FDCE  PORT MAP ( C=>C, CE=>Hi, CLR=>Low, D=>I,  Q=>Ta );
U1:    FDCE  PORT MAP ( C=>C, CE=>Hi, CLR=>Low, D=>Ta, Q=>Tb );
U2:    FDCE  PORT MAP ( C=>C, CE=>Hi, CLR=>Low, D=>Tc, Q=>Q  );
--
END         MacroCellRE;
--
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.arch.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY      aFallingEdge IS
--          ~~~~~~~~~~~~
        PORT (
             I:     IN     STD_LOGIC;
             C:     IN     STD_LOGIC;
--
             Q:     OUT    STD_LOGIC ); -- Out
END         aFallingEdge;
--
ARCHITECTURE MacroCellFE  OF aFallingEdge IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SIGNAL       Ta,Tb,Tc    : STD_LOGIC;
SIGNAL       Low,Hi      : STD_LOGIC;
--
BEGIN
--
Low <= '0';
Hi  <= '1';
Tc  <= NOT I AND NOT Ta AND Tb;
--
U0:    FDCE  PORT MAP ( C=>C, CE=>Hi, CLR=>Low, D=>I,  Q=>Ta );
U1:    FDCE  PORT MAP ( C=>C, CE=>Hi, CLR=>Low, D=>Ta, Q=>Tb );
U2:    FDCE  PORT MAP ( C=>C, CE=>Hi, CLR=>Low, D=>Tc, Q=>Q  );
--
END         MacroCellFE;
--
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE IEEE.MATH_REAL.ALL;
USE work.tuw.all;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
--
ENTITY aDelay IS
--
	GENERIC(
		Length: INTEGER := 1
	);
	PORT (
		I:	IN	STD_LOGIC;  -- In
		C:	IN	STD_LOGIC;  -- Clock
		CE:	IN	STD_LOGIC;  -- Clock Enable
		Q:	OUT	STD_LOGIC
	); -- Out
END         aDelay;
--
ARCHITECTURE LUTCellDL  OF aDelay   IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~
CONSTANT NumOfSRLs	: INTEGER	:= INTEGER(CEIL(REAL(Length)/16.0));

SIGNAL A			: STD_LOGIC_VECTOR ( 03					downto 00 );

BEGIN
--
--
		Let( A, Length-1);
		
		U0: SRL16E GENERIC MAP (
			INIT =>x"0000"
		)
		PORT MAP (
			D	=>I,
			CE	=>CE,
			CLK	=>C,
			A0=>A(0),A1=>A(1),A2=>A(2),A3=>A(3),
			Q=>Q );
	
END  LUTCellDL;

library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE IEEE.MATH_REAL.ALL;
USE work.tuw.all;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
--
ENTITY aDelayx2 IS
--
	GENERIC(
		Length: INTEGER := 1
	);
	PORT (
		I:	IN	STD_LOGIC;  -- In
		C:	IN	STD_LOGIC;  -- Clock
		CE:	IN	STD_LOGIC;  -- Clock Enable
		Q:	OUT	STD_LOGIC
	); -- Out
END         aDelayx2;
--
ARCHITECTURE LUTCellDL  OF aDelayx2   IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~

SIGNAL A			: STD_LOGIC_VECTOR ( 04					downto 00 );

BEGIN
--
--
		Let( A, Length-1);
		
		U0: SRLC32E GENERIC MAP (
			INIT => x"00000000"
		)
		PORT MAP (
			D	=>I,
			CE	=>CE,
			CLK	=>C,
			A	=>A,
			Q	=>Q 
		);
	
END  LUTCellDL;
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.tuw.ALL;
USE work.arch.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY aPipe IS
--          ~~~~~
	GENERIC(
		Size		: INTEGER := aDefSize;
		Length	: INTEGER := 1
	);
	PORT (
		I	: IN		STD_LOGIC_VECTOR (Size-1 downto 0);  -- IN
		C	: IN		STD_LOGIC;         -- Clock
		CE	: IN		STD_LOGIC;         -- Clock Enable
		Q15	: OUT	STD_LOGIC_VECTOR (Size-1 downto 0);
		Q	: OUT	STD_LOGIC_VECTOR (Size-1 downto 0)
	); -- OUT
END aPipe;
--
ARCHITECTURE LUTCellPP  OF aPipe    IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~
SIGNAL	A : STD_LOGIC_VECTOR (3 downto 0);
BEGIN
--
Let( A, Length-1 );
--
G0: FOR J IN 0 TO Size-1 GENERATE
	U0: SRLC16E GENERIC MAP (
		INIT=> x"0000"
	)
	PORT MAP (
		D=>I(J), CE=>CE, CLK=>C,
		A0=>A(0),A1=>A(1),A2=>A(2),A3=>A(3),
		Q=>Q(J),Q15=>Q15(J)
	);
END GENERATE;
--
END  LUTCellPP;
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.tuw.ALL;
USE work.arch.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY aPipex2 IS
--          ~~~~~
	GENERIC(
		Size		: INTEGER := aDefSize;
		Length	: INTEGER := 1
	);
	PORT (
		I	: IN		STD_LOGIC_VECTOR (Size-1 downto 0);  -- IN
		C	: IN		STD_LOGIC;         -- Clock
		CE	: IN		STD_LOGIC;         -- Clock Enable
		Q31	: OUT	STD_LOGIC_VECTOR (Size-1 downto 0);
		Q	: OUT	STD_LOGIC_VECTOR (Size-1 downto 0)
	); -- OUT
END aPipex2;
--
ARCHITECTURE LUTCellPP  OF aPipex2    IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~
SIGNAL	A : STD_LOGIC_VECTOR (5-1 downto 0);
BEGIN
--
Let( A, Length-1 );
--
G0: FOR J IN 0 TO Size-1 GENERATE
	U0: SRLC32E GENERIC MAP (
		INIT=> x"00000000"
	)
	PORT MAP (
		D  	=>I(J), 
		CE 	=>CE, 
		CLK 	=>C,
		A 	=> A,
		Q 	=>Q(J),
		Q31 	=>Q31(J)
	);
END GENERATE;
--
END  LUTCellPP;
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.tuw.ALL;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY      aProgPipe   IS
--          ~~~~~~~~~
        GENERIC( Size:   INTEGER := aDefSize );
        PORT (
             I:     IN     STD_LOGIC_VECTOR (Size-1 downto 0);  -- In
             C:     IN     STD_LOGIC;         -- Clock
             CE:    IN     STD_LOGIC;         -- Clock Enable
             A:     IN     STD_LOGIC_VECTOR (3 downto 0);       -- Addr.
             Q:     OUT    STD_LOGIC_VECTOR (Size-1 downto 0)); -- Out
END         aProgPipe;
--
ARCHITECTURE LUTCellPPP  OF aProgPipe    IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~
BEGIN
--
G0: FOR J IN 0 TO Size-1 GENERATE
U0: SRL16E 
GENERIC MAP (
	INIT =>x"0000" )
PORT MAP (
	D		=>I(J), 
	CE		=>CE, 
	CLK		=>C,
	A0		=>A(0),
	A1		=>A(1),
	A2		=>A(2),
	A3		=>A(3),
	Q		=>Q(J) 
	  );
END GENERATE;
--
END  LUTCellPPP;
--
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.arch.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY      aRippleRise IS
--          ~~~~~~~~~~~
        PORT (
             I:     IN     STD_LOGIC;
             C:     IN     STD_LOGIC;
             Q:     OUT    STD_LOGIC
             );
END         aRippleRise;
--
ARCHITECTURE MacroCellRR  OF aRippleRise IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~~
SIGNAL       Ta,Tp       : STD_LOGIC;
SIGNAL       Low,Hi      : STD_LOGIC;
--
BEGIN
--
Low <= '0';
Hi  <= '1';
Q   <= Tp;
--
U0:    FDCE  PORT MAP ( C=>I, CE=>Hi, CLR=>Tp,  D=>Hi, Q=>Ta );
U1:    FDCE  PORT MAP ( C=>C, CE=>Hi, CLR=>Low, D=>Ta, Q=>Tp );
--
END         MacroCellRR;
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.arch.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY      aRippleRiseSync IS
--          ~~~~~~~~~~~~~~~
        PORT (
             I:     IN     STD_LOGIC;       -- Input sync. other clock
             CI:    IN     STD_LOGIC;       -- Clock In
             Q:     OUT    STD_LOGIC;       -- Pulse on Rising edge
             CO:    IN     STD_LOGIC        -- Clock Out
             );
END         aRippleRiseSync;
--
ARCHITECTURE MacroCellRR  OF aRippleRiseSync IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SIGNAL       Ta,Tp       : STD_LOGIC;
SIGNAL       Low,Hi      : STD_LOGIC;
--
BEGIN
--
Low <= '0';
Hi  <= '1';
Q   <= Tp;
--
U0:    FDCE  PORT MAP ( C=>CI, CE=>I,  CLR=>Tp,  D=>Hi, Q=>Ta );
U1:    FDCE  PORT MAP ( C=>CO, CE=>Hi, CLR=>Low, D=>Ta, Q=>Tp );
--
END         MacroCellRR;
--
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.arch.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY      aRippleFall  IS
--          ~~~~~~~~~~~
        PORT (
             I:     IN     STD_LOGIC;
             C:     IN     STD_LOGIC;
--
             Q:     OUT    STD_LOGIC ); -- Out
END         aRippleFall;
--
ARCHITECTURE MacroCellRF  OF aRippleFall  IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~~
SIGNAL       Ta,Tp       : STD_LOGIC;
SIGNAL       Low,Hi      : STD_LOGIC;
--
BEGIN
--
Low <= '0';
Hi  <= '1';
Q   <= Tp;
--
U0:    FDCE_1  PORT MAP ( C=>I, CE=>Hi, CLR=>Tp,  D=>Hi, Q=>Ta );
U1:    FDCE    PORT MAP ( C=>C, CE=>Hi, CLR=>Low, D=>Ta, Q=>Tp );
--
END         MacroCellRF;
--
--
library IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE work.arch.all;
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
USE work.arch.aDefSize;
--
ENTITY      aRegister   IS
--          ~~~~~~~~~
        GENERIC( Size:   INTEGER := aDefSize );
        PORT (
             D:     IN     STD_LOGIC_VECTOR (Size-1 downto 0);  -- In
             C:     IN     STD_LOGIC;   -- Clock
             CE:    IN     STD_LOGIC;   -- Clock Enable
--
             Q:     OUT    STD_LOGIC_VECTOR (Size-1 downto 0)); -- Out
END         aRegister;
--
ARCHITECTURE MacroCellre  OF aRegister  IS
--           ~~~~~~~~~~~~~~~~~~~~~~~~~~
BEGIN
--
G1: FOR J IN 0 TO Size-1 GENERATE
	U1: FDCPE GENERIC MAP (INIT=>'0') PORT MAP ( D=>D(J), C=>C, CE=>CE,CLR=>'0',PRE=>'0', Q=>Q(J) ); 

END GENERATE;
--
END  MacroCellre;
--
--
library IEEE;
use IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.ALL;
use work.tuw.all;
use work.arch.aDefSize; --all;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
--
ENTITY aRAM   IS
--     ~~~~~~
GENERIC( Size: INTEGER := aDefSize );
PORT (
     D     : IN  STD_LOGIC_VECTOR( Size-1 downto 0 ); -- Data in
     WE    : IN  STD_LOGIC;       -- Write enable
     WCLK  : IN  STD_LOGIC;       -- Clock
     A     : IN  STD_LOGIC_VECTOR ( 3 downto 0);      -- Write Address
--
     RA    : IN  STD_LOGIC_VECTOR ( 3 downto 0);      -- Read Address
     DO    : OUT STD_LOGIC_VECTOR( Size-1 downto 0 )  -- Data out
     );
END aRAM;
--
ARCHITECTURE LUTRAM33 OF aRAM IS
--           =================
BEGIN
--
G1: FOR I IN 0 TO Size-1 GENERATE
--
U1: RAM16X1D    PORT    MAP(
-- -----------
                              D     => D(I),          -- Write data
                              WE    => WE,            -- Write enable
                              WCLK  => WCLK,          -- Clock
                              A0    => A(0),          -- Write Address
                              A1    => A(1),          -- Write Address
                              A2    => A(2),          -- Write Address
                              A3    => A(3),          -- Write Address
                              DPRA0 => RA(0),         -- Read address
                              DPRA1 => RA(1),         -- Read address
                              DPRA2 => RA(2),         -- Read address
                              DPRA3 => RA(3),         -- Read address
                              DPO   => DO(I) );       -- Data out
END GENERATE;
END LUTRAM33;
----
library IEEE;
use IEEE.std_logic_1164.all;
USE IEEE.std_logic_UNSIGNED.ALL;
use work.tuw.all;
use work.arch.all;
--
ENTITY aRAMx2 IS
--     ~~~~~~
GENERIC( Size: INTEGER := aDefSize );
PORT (
     D     : IN  STD_LOGIC_VECTOR( Size-1 downto 0 ); -- Data in
     WE    : IN  STD_LOGIC;       -- Write enable
     WCLK  : IN  STD_LOGIC;       -- Clock
     A     : IN  STD_LOGIC_VECTOR ( 4 downto 0);      -- Write Address
--
     RA    : IN  STD_LOGIC_VECTOR ( 4 downto 0);      -- Read Address
     DO    : OUT STD_LOGIC_VECTOR( Size-1 downto 0 )  -- Data out
     );
END aRAMx2;
--
ARCHITECTURE LUTRAM33x2 OF aRAMx2 IS
--           ====================
   SIGNAL   Wr1,Wr2     : STD_LOGIC;
   SIGNAL   Dout1,Dout2 : STD_LOGIC_VECTOR( Size-1 downto 0);
--
BEGIN
--
Wr1 <= WE AND NOT A(4);
Wr2 <= WE AND     A(4);
--
U1: aRAM        GENERIC MAP(  Size  => Size )
-- -----------
                PORT    MAP(  D     => D,               -- Write data
                              WE    => Wr1,             -- Write enable
                              WCLK  => WCLK,            -- Clock
                              A     => A(3 downto 0),   -- Write Address
                              RA    => RA(3 downto 0),  -- Read address
                              DO    => Dout1  );        -- Data out
--
U2: aRAM        GENERIC MAP(  Size  => Size )
-- -----------
                PORT    MAP(  D     => D,               -- Write data
                              WE    => Wr2,             -- Write enable
                              WCLK  => WCLK,            -- Clock
                              A     => A(3 downto 0),   -- Write Address
                              RA    => RA(3 downto 0),  -- Read address
                              DO    => Dout2  );        -- Data out
WITH RA(4) SELECT DO <=  Dout1 WHEN '0',
                         Dout2 WHEN OTHERS;
--
END LUTRAM33x2;

-- ******** end of arch.vhd ***********
